//=============================================================================
//
// file :        ABending.h
//
// description : Include for the ABending class.
//
// project :	Class representing particel accelerator bending
//
// $Author: claudio $
//
// $Revision: 1.11 $
// $Date: 2018-08-28 13:32:12 $
//
// SVN only:
// $HeadURL: $
//
// CVS only:
// $Source: /home/cvsadm/cvsroot/fermi/servers/bending/src/ABending.h,v $
// $Log: ABending.h,v $
// Revision 1.11  2018-08-28 13:32:12  claudio
// TANGO_BASE_CLASS, #4939: 0 field if OFF/FAULT/STANDBY
//
// Revision 1.10  2017-08-17 12:43:27  claudio
// #1554 tango-9.2.5a
//
// Revision 1.9  2012-06-05 13:11:00  claudio
// added ExcitationSetPoint attribute
//
// Revision 1.8  2012-06-04 15:48:24  claudio
// read e setting separati
// piu' robusta gestione eccezzioni
//
// Revision 1.7  2012-02-29 13:00:42  claudio
// ticket 891:  aggiunti attributi per leggere minimi e massimi delle grandezze impostabili. Vengono ricalcolati quando cambia l'energia.
//
// Revision 1.6  2012-01-31 11:30:45  claudio
// ticket 876: problema di eccezioni mal gestite. risolto
// aggiunta update anche in always_executed()
// messo a fattor comune il metodo di update
//
// Revision 1.5  2010/11/17 14:10:21  claudio
// remeoved unused vars.
// starts also with power supply devices down
//
// Revision 1.4  2010/09/10 09:34:10  claudio
// handle also power supply (anomalous) with DevFloat attribute
// (see ticket 356)
//
// Revision 1.3  2010/09/02 14:14:34  claudio
// improved handling of device state:
// - mirrors power supply state
// - fault if cannto read power supply
// - is OFF or STANDBY force 0 reading and setting
// - rationalized handling of update values
//
// Revision 1.2  2010/03/31 15:38:08  claudio
// added Kick in milliradians
//
// Revision 1.1  2010/02/23 15:45:19  claudio
// bending magnet model - first import
//
//
// copyleft :    European Synchrotron Radiation Facility
//               BP 220, Grenoble 38043
//               FRANCE
//
//=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================
#ifndef _ABENDING_H
#define _ABENDING_H

#include <tango.h>
//using namespace Tango;
#include <magnetfactory.h>

/**
 * @author	$Author: claudio $
 * @version	$Revision: 1.11 $
 */

 //	Add your own constant definitions here.
 //-----------------------------------------------


namespace ABending_ns
{
using namespace magnetfield;
/**
 * Class Description:
 * Tango device for manipulating accelerator magnets in terms of machine physics quantities.
 *	The SETTING of the excitation drives all the conversions and readings.
 *	That is: noise and fluctutations of the read-out are ignored.
 *	The magnet model is taken from Elettra's Hlf class library.
 */

/*
 *	Device States Description:
*  Tango::ON :       The magnet is up and working
*  Tango::OFF :      The magnet  is off : no field and not operable
*  Tango::UNKNOWN :  The underliyng power supply or communication is broken
*  Tango::STANDBY :  The underlying power_supply is in standby
*  Tango::FAULT :    The underlying power supply ins in FAULT
*  Tango::RUNNING :  The underlying power supply ins in ramping or cycling
*  Tango::ALARM :
 */


class ABending: public TANGO_BASE_CLASS
{
public :
	//	Add your own data members here
	//-----------------------------------------


	//	Here is the Start of the automatic code generation part
	//-------------------------------------------------------------	
/**
 *	@name attributes
 *	Attribute member data.
 */
//@{
		Tango::DevDouble	*attr_Excitation_read;
		Tango::DevDouble	attr_Excitation_write;
		Tango::DevDouble	*attr_ExcitationSetPoint_read;
		Tango::DevDouble	*attr_MagnetEnergy_read;
		Tango::DevDouble	attr_MagnetEnergy_write;
		Tango::DevDouble	*attr_MagnetEnergySetPoint_read;
		Tango::DevDouble	*attr_Energy_read;
		Tango::DevDouble	attr_Energy_write;
		Tango::DevDouble	*attr_Kick_read;
		Tango::DevDouble	attr_Kick_write;
		Tango::DevDouble	*attr_KickSetPoint_read;
		Tango::DevDouble	*attr_MinMagnetEnergy_read;
		Tango::DevDouble	*attr_MaxMagnetEnergy_read;
		Tango::DevDouble	*attr_MinKick_read;
		Tango::DevDouble	*attr_MaxKick_read;
//@}

/**
 * @name Device properties
 * Device properties member data.
 */
//@{
/**
 *	The Database host name
 */
	string	dbName;
/**
 *	The Database host name
 */
	string	dbHost;
/**
 *	The MySQL password
 */
	string	dbPassword;
/**
 *	The MySQL database user name
 */
	string	dbUser;
/**
 *	The name of the magnet, for instance `QD_B'
 *	The name is used for database loopkup for getting calibration tables.
 */
	string	magnetName;
/**
 *	The type of the magnet, must be one of the following:
 *	DIPOLE
 *	SECTORDIPOLE
 */
	string	magnetType;
/**
 *	The name of the power supply associated to the magnet.
 *	For instance `b/power_supply/psqd_b'
 */
	string	powerSupply;
/**
 *	This is the name of the attribute that returns the excitation of the power supply.
 *	For instance `Current' or `Voltage'
 */
	string	pSExcitationAttributeName;
/**
 *	minimum time interval between two reading of the excitation. In milliseconds.
 *	The purpose is to limit the number of calls to the real hardware
 */
	Tango::DevULong	readInterval;
/**
 *	Specify which sign convention to use for horizaontally focussing elements
 *	if True : focussing -> K>0 ( MAD, elegant,... )
 *	if False : focussing -> K<0 ( vicky )
 */
	Tango::DevBoolean	madSignConvention;
//@}

/**
 *	@name Device properties
 *	Device property member data.
 */
//@{
//@}

/**@name Constructors
 * Miscellaneous constructors */
//@{
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	ABending(Tango::DeviceClass *cl,string &s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	ABending(Tango::DeviceClass *cl,const char *s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device name
 *	@param d	Device description.
 */
	ABending(Tango::DeviceClass *cl,const char *s,const char *d);
//@}

/**@name Destructor
 * Only one destructor is defined for this class */
//@{
/**
 * The object destructor.
 */	
	~ABending() {delete_device();};
/**
 *	will be called at device destruction or at init command.
 */
	void delete_device();
//@}

	
/**@name Miscellaneous methods */
//@{
/**
 *	Initialize the device
 */
	virtual void init_device();
/**
 *	Always executed method before execution command method.
 */
	virtual void always_executed_hook();

//@}

/**
 * @name ABending methods prototypes
 */

//@{
/**
 *	Hardware acquisition for attributes.
 */
	virtual void read_attr_hardware(vector<long> &attr_list);
/**
 *	Extract real attribute values for Excitation acquisition result.
 */
	virtual void read_Excitation(Tango::Attribute &attr);
/**
 *	Write Excitation attribute values to hardware.
 */
	virtual void write_Excitation(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for ExcitationSetPoint acquisition result.
 */
	virtual void read_ExcitationSetPoint(Tango::Attribute &attr);
/**
 *	Extract real attribute values for MagnetEnergy acquisition result.
 */
	virtual void read_MagnetEnergy(Tango::Attribute &attr);
/**
 *	Write MagnetEnergy attribute values to hardware.
 */
	virtual void write_MagnetEnergy(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for MagnetEnergySetPoint acquisition result.
 */
	virtual void read_MagnetEnergySetPoint(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Energy acquisition result.
 */
	virtual void read_Energy(Tango::Attribute &attr);
/**
 *	Write Energy attribute values to hardware.
 */
	virtual void write_Energy(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for Kick acquisition result.
 */
	virtual void read_Kick(Tango::Attribute &attr);
/**
 *	Write Kick attribute values to hardware.
 */
	virtual void write_Kick(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for KickSetPoint acquisition result.
 */
	virtual void read_KickSetPoint(Tango::Attribute &attr);
/**
 *	Extract real attribute values for MinMagnetEnergy acquisition result.
 */
	virtual void read_MinMagnetEnergy(Tango::Attribute &attr);
/**
 *	Extract real attribute values for MaxMagnetEnergy acquisition result.
 */
	virtual void read_MaxMagnetEnergy(Tango::Attribute &attr);
/**
 *	Extract real attribute values for MinKick acquisition result.
 */
	virtual void read_MinKick(Tango::Attribute &attr);
/**
 *	Extract real attribute values for MaxKick acquisition result.
 */
	virtual void read_MaxKick(Tango::Attribute &attr);
/**
 *	Read/Write allowed for Excitation attribute.
 */
	virtual bool is_Excitation_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for ExcitationSetPoint attribute.
 */
	virtual bool is_ExcitationSetPoint_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for MagnetEnergy attribute.
 */
	virtual bool is_MagnetEnergy_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for MagnetEnergySetPoint attribute.
 */
	virtual bool is_MagnetEnergySetPoint_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Energy attribute.
 */
	virtual bool is_Energy_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Kick attribute.
 */
	virtual bool is_Kick_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for KickSetPoint attribute.
 */
	virtual bool is_KickSetPoint_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for MinMagnetEnergy attribute.
 */
	virtual bool is_MinMagnetEnergy_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for MaxMagnetEnergy attribute.
 */
	virtual bool is_MaxMagnetEnergy_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for MinKick attribute.
 */
	virtual bool is_MinKick_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for MaxKick attribute.
 */
	virtual bool is_MaxKick_allowed(Tango::AttReqType type);

/**
 *	Read the device properties from database
 */
	 void get_device_property();
//@}

	//	Here is the end of the automatic code generation part
	//-------------------------------------------------------------	



protected :	
	//	Add your own data members here
	Dipole *magnet;
	double excitation_set_point;
	double excitation_read;
	double magnet_energy_set_point;
	double magnet_energy_read;
	double kick_set_point;
	double kick_read;
	string _name_of_type;
	double energy;
	//! Wattribute references for updating the write part of attributes on init, etc
	Tango::WAttribute *attr_magnet_energy, *attr_excitation, *attr_energy, *attr_kick ;
	Tango::Attribute *attr_ps;
	void GetExcitationFromPowerSupply();
	void SetExcitationOnPowerSupply(double excitation);
        double min_ps,max_ps; //!< power_supply limits
        double min_ex,max_ex; //!< excitation limits
        double min_kick,max_kick;   //!< kick limits
        double min_me,max_me;//!< magnet energy limits
	//-----------------------------------------
	
private:
	Magnetfactory* mf;
	Tango::DeviceProxy *dev;
	bool use_double_attribute;
	
	//Tango::DevDouble energy, strength, field,excitation;
	
	Dipole *CreateMagnet(string name);
	
	void build_DevFailed_and_throw(const char* problem, const char *desc, const char* origin);
		
	bool first_read;

	
	int delta_milliseconds(struct timeval *t1, struct timeval *t2);
	struct timeval read_stamp;
	Tango::DevState powersupply_state;
        void updateInternalValues(void);
        void read_and_update(void);
        void setStrengthLimits(void);
};

}	// namespace_ns

#endif	// _ABENDING_H
