static const char *ClassId    = "$Id: ABendingClass.cpp,v 1.7 2018-08-28 13:32:12 claudio Exp $";
static const char *TagName    = "$Name:  $";
static const char *CvsPath    = "$Source: /home/cvsadm/cvsroot/fermi/servers/bending/src/ABendingClass.cpp,v $";
static const char *SvnPath    = "$HeadURL: $";
static const char *HttpServer = "http://www.esrf.fr/computing/cs/tango/tango_doc/ds_doc/";
//+=============================================================================
//
// file :        ABendingClass.cpp
//
// description : C++ source for the ABendingClass. A singleton
//               class derived from DeviceClass. It implements the
//               command list and all properties and methods required
//               by the ABending once per process.
//
// project :     TANGO Device Server
//
// $Author: claudio $
//
// $Revision: 1.7 $
//
// $Log: ABendingClass.cpp,v $
// Revision 1.7  2018-08-28 13:32:12  claudio
// TANGO_BASE_CLASS, #4939: 0 field if OFF/FAULT/STANDBY
//
// Revision 1.6  2012-06-05 13:11:00  claudio
// added ExcitationSetPoint attribute
//
// Revision 1.5  2012-06-04 15:48:24  claudio
// read e setting separati
// piu' robusta gestione eccezzioni
//
// Revision 1.4  2012-02-29 13:00:42  claudio
// ticket 891:  aggiunti attributi per leggere minimi e massimi delle grandezze impostabili. Vengono ricalcolati quando cambia l'energia.
//
// Revision 1.3  2010/09/02 14:14:34  claudio
// improved handling of device state:
// - mirrors power supply state
// - fault if cannto read power supply
// - is OFF or STANDBY force 0 reading and setting
// - rationalized handling of update values
//
// Revision 1.2  2010/03/31 15:38:08  claudio
// added Kick in milliradians
//
// Revision 1.1  2010/02/23 15:45:19  claudio
// bending magnet model - first import
//
//
// copyleft :   European Synchrotron Radiation Facility
//              BP 220, Grenoble 38043
//              FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================


#include <tango.h>

#include <ABending.h>
#include <ABendingClass.h>


//+----------------------------------------------------------------------------
/**
 *	Create ABendingClass singleton and return it in a C function for Python usage
 */
//+----------------------------------------------------------------------------
extern "C" {
#ifdef WIN32

__declspec(dllexport)

#endif

	Tango::DeviceClass *_create_ABending_class(const char *name) {
		return ABending_ns::ABendingClass::init(name);
	}
}


namespace ABending_ns
{


//
//----------------------------------------------------------------
//	Initialize pointer for singleton pattern
//----------------------------------------------------------------
//
ABendingClass *ABendingClass::_instance = NULL;

//+----------------------------------------------------------------------------
//
// method : 		ABendingClass::ABendingClass(string &s)
// 
// description : 	constructor for the ABendingClass
//
// in : - s : The class name
//
//-----------------------------------------------------------------------------
ABendingClass::ABendingClass(string &s):DeviceClass(s)
{

	cout2 << "Entering ABendingClass constructor" << endl;
	set_default_property();
	get_class_property();
	write_class_property();
	
	cout2 << "Leaving ABendingClass constructor" << endl;

}
//+----------------------------------------------------------------------------
//
// method : 		ABendingClass::~ABendingClass()
// 
// description : 	destructor for the ABendingClass
//
//-----------------------------------------------------------------------------
ABendingClass::~ABendingClass()
{
	_instance = NULL;
}

//+----------------------------------------------------------------------------
//
// method : 		ABendingClass::instance
// 
// description : 	Create the object if not already done. Otherwise, just
//			return a pointer to the object
//
// in : - name : The class name
//
//-----------------------------------------------------------------------------
ABendingClass *ABendingClass::init(const char *name)
{
	if (_instance == NULL)
	{
		try
		{
			string s(name);
			_instance = new ABendingClass(s);
		}
		catch (bad_alloc)
		{
			throw;
		}		
	}		
	return _instance;
}

ABendingClass *ABendingClass::instance()
{
	if (_instance == NULL)
	{
		cerr << "Class is not initialised !!" << endl;
		exit(-1);
	}
	return _instance;
}

//+----------------------------------------------------------------------------
//
// method : 		ABendingClass::command_factory
// 
// description : 	Create the command object(s) and store them in the 
//			command list
//
//-----------------------------------------------------------------------------
void ABendingClass::command_factory()
{

	//	add polling if any
	for (unsigned int i=0 ; i<command_list.size(); i++)
	{
	}
}

//+----------------------------------------------------------------------------
//
// method : 		ABendingClass::get_class_property
// 
// description : 	Get the class property for specified name.
//
// in :		string	name : The property name
//
//+----------------------------------------------------------------------------
Tango::DbDatum ABendingClass::get_class_property(string &prop_name)
{
	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
		if (cl_prop[i].name == prop_name)
			return cl_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}
//+----------------------------------------------------------------------------
//
// method : 		ABendingClass::get_default_device_property()
// 
// description : 	Return the default value for device property.
//
//-----------------------------------------------------------------------------
Tango::DbDatum ABendingClass::get_default_device_property(string &prop_name)
{
	for (unsigned int i=0 ; i<dev_def_prop.size() ; i++)
		if (dev_def_prop[i].name == prop_name)
			return dev_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//+----------------------------------------------------------------------------
//
// method : 		ABendingClass::get_default_class_property()
// 
// description : 	Return the default value for class property.
//
//-----------------------------------------------------------------------------
Tango::DbDatum ABendingClass::get_default_class_property(string &prop_name)
{
	for (unsigned int i=0 ; i<cl_def_prop.size() ; i++)
		if (cl_def_prop[i].name == prop_name)
			return cl_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}
//+----------------------------------------------------------------------------
//
// method : 		ABendingClass::device_factory
// 
// description : 	Create the device object(s) and store them in the 
//			device list
//
// in :		Tango::DevVarStringArray *devlist_ptr : The device name list
//
//-----------------------------------------------------------------------------
void ABendingClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
{

	//	Create all devices.(Automatic code generation)
	//-------------------------------------------------------------
	for (unsigned long i=0 ; i < devlist_ptr->length() ; i++)
	{
		cout4 << "Device name : " << (*devlist_ptr)[i].in() << endl;
						
		// Create devices and add it into the device list
		//----------------------------------------------------
		device_list.push_back(new ABending(this, (*devlist_ptr)[i]));							 

		// Export device to the outside world
		// Check before if database used.
		//---------------------------------------------
		if ((Tango::Util::_UseDb == true) && (Tango::Util::_FileDb == false))
			export_device(device_list.back());
		else
			export_device(device_list.back(), (*devlist_ptr)[i]);
	}
	//	End of Automatic code generation
	//-------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//	Method: ABendingClass::attribute_factory(vector<Tango::Attr *> &att_list)
//-----------------------------------------------------------------------------
void ABendingClass::attribute_factory(vector<Tango::Attr *> &att_list)
{
	//	Attribute : Excitation
	ExcitationAttrib	*excitation = new ExcitationAttrib();
	Tango::UserDefaultAttrProp	excitation_prop;
	excitation_prop.set_label("Excitation");
	excitation_prop.set_unit("A");
	excitation_prop.set_standard_unit("1");
	excitation_prop.set_display_unit("A");
	excitation_prop.set_format("%6.3f");
	excitation_prop.set_description("The magnet excitation (V or I depending on power supply type ).");
	excitation->set_default_properties(excitation_prop);
	att_list.push_back(excitation);

	//	Attribute : ExcitationSetPoint
	ExcitationSetPointAttrib	*excitation_set_point = new ExcitationSetPointAttrib();
	Tango::UserDefaultAttrProp	excitation_set_point_prop;
	excitation_set_point_prop.set_label("Excitation set point");
	excitation_set_point_prop.set_unit("A");
	excitation_set_point_prop.set_standard_unit("1");
	excitation_set_point_prop.set_display_unit("A");
	excitation_set_point_prop.set_format("%6.3f");
	excitation_set_point_prop.set_description("The magnet excitation set point");
	excitation_set_point->set_default_properties(excitation_set_point_prop);
	att_list.push_back(excitation_set_point);

	//	Attribute : MagnetEnergy
	MagnetEnergyAttrib	*magnet_energy = new MagnetEnergyAttrib();
	Tango::UserDefaultAttrProp	magnet_energy_prop;
	magnet_energy_prop.set_label("Energy");
	magnet_energy_prop.set_unit("GeV");
	magnet_energy_prop.set_standard_unit("1");
	magnet_energy_prop.set_display_unit("GeV");
	magnet_energy_prop.set_format("%5.3f");
	magnet_energy_prop.set_description("Bending nominal energy\nWhen you write the MagnetEnery, the excitation is calculated and applied to the power supply pointed to by the device property `PowerSupply'.\nThis attribute is available only for DIPOLE");
	magnet_energy->set_default_properties(magnet_energy_prop);
	att_list.push_back(magnet_energy);

	//	Attribute : MagnetEnergySetPoint
	MagnetEnergySetPointAttrib	*magnet_energy_set_point = new MagnetEnergySetPointAttrib();
	Tango::UserDefaultAttrProp	magnet_energy_set_point_prop;
	magnet_energy_set_point_prop.set_label("Energy set point");
	magnet_energy_set_point_prop.set_unit("GeV");
	magnet_energy_set_point_prop.set_standard_unit("1");
	magnet_energy_set_point_prop.set_display_unit("GeV");
	magnet_energy_set_point_prop.set_format("%5.3f");
	magnet_energy_set_point_prop.set_description("Bending nominal energy, calculated from the set point of the power supply.\nThis attribute is available only for DIPOLE");
	magnet_energy_set_point->set_default_properties(magnet_energy_set_point_prop);
	att_list.push_back(magnet_energy_set_point);

	//	Attribute : Energy
	EnergyAttrib	*energy = new EnergyAttrib();
	Tango::UserDefaultAttrProp	energy_prop;
	energy_prop.set_label("Beam Energy");
	energy_prop.set_unit("GeV");
	energy_prop.set_standard_unit("1");
	energy_prop.set_display_unit("GeV");
	energy_prop.set_format("%6.3f");
	energy_prop.set_min_value("0.001");
	energy_prop.set_description("The effective energy, in GeV, of the electron beam.\nThis is a secondary parameter used interanally by the magnet model to perform some calculations.");
	energy->set_default_properties(energy_prop);
	att_list.push_back(energy);

	//	Attribute : Kick
	KickAttrib	*kick = new KickAttrib();
	Tango::UserDefaultAttrProp	kick_prop;
	kick_prop.set_label("Deflection Angle");
	kick_prop.set_unit("mrad");
	kick_prop.set_standard_unit("1");
	kick_prop.set_display_unit("mrad");
	kick_prop.set_format("%5.4f");
	kick_prop.set_description("Magnet kick in radians.\nWhen you write the kick, the excitation is calculated and applied to the power supply pointed to by the device property `PowerSupply'.\nThis attribute is available only for DIPOLE and KICKER magnets");
	kick->set_default_properties(kick_prop);
	att_list.push_back(kick);

	//	Attribute : KickSetPoint
	KickSetPointAttrib	*kick_set_point = new KickSetPointAttrib();
	Tango::UserDefaultAttrProp	kick_set_point_prop;
	kick_set_point_prop.set_label("Deflection Angle set soint");
	kick_set_point_prop.set_unit("mrad");
	kick_set_point_prop.set_standard_unit("1");
	kick_set_point_prop.set_display_unit("mrad");
	kick_set_point_prop.set_format("%5.4f");
	kick_set_point_prop.set_description("Magnet kick in radians, calculated from power supply set point end beam energy.\nThis attribute is available only for DIPOLE and KICKER magnets");
	kick_set_point->set_default_properties(kick_set_point_prop);
	att_list.push_back(kick_set_point);

	//	Attribute : MinMagnetEnergy
	MinMagnetEnergyAttrib	*min_magnet_energy = new MinMagnetEnergyAttrib();
	Tango::UserDefaultAttrProp	min_magnet_energy_prop;
	min_magnet_energy_prop.set_label("min. Energy");
	min_magnet_energy_prop.set_unit("GeV");
	min_magnet_energy_prop.set_standard_unit("1");
	min_magnet_energy_prop.set_display_unit("GeV");
	min_magnet_energy_prop.set_format("%5.3f");
	min_magnet_energy_prop.set_description("Minimum bending nominal energy");
	min_magnet_energy->set_default_properties(min_magnet_energy_prop);
	att_list.push_back(min_magnet_energy);

	//	Attribute : MaxMagnetEnergy
	MaxMagnetEnergyAttrib	*max_magnet_energy = new MaxMagnetEnergyAttrib();
	Tango::UserDefaultAttrProp	max_magnet_energy_prop;
	max_magnet_energy_prop.set_label("max. Energy");
	max_magnet_energy_prop.set_unit("GeV");
	max_magnet_energy_prop.set_standard_unit("1");
	max_magnet_energy_prop.set_display_unit("GeV");
	max_magnet_energy_prop.set_format("%5.3f");
	max_magnet_energy_prop.set_description("Maximum bending nominal energy");
	max_magnet_energy->set_default_properties(max_magnet_energy_prop);
	att_list.push_back(max_magnet_energy);

	//	Attribute : MinKick
	MinKickAttrib	*min_kick = new MinKickAttrib();
	Tango::UserDefaultAttrProp	min_kick_prop;
	min_kick_prop.set_label("min. Deflection Angle");
	min_kick_prop.set_unit("mrad");
	min_kick_prop.set_standard_unit("1");
	min_kick_prop.set_display_unit("rad");
	min_kick_prop.set_format("%5.4f");
	min_kick_prop.set_description("Minimum magnet kick in radians\n");
	min_kick->set_default_properties(min_kick_prop);
	att_list.push_back(min_kick);

	//	Attribute : MaxKick
	MaxKickAttrib	*max_kick = new MaxKickAttrib();
	Tango::UserDefaultAttrProp	max_kick_prop;
	max_kick_prop.set_label("max. Deflection Angle");
	max_kick_prop.set_unit("mrad");
	max_kick_prop.set_standard_unit("1");
	max_kick_prop.set_display_unit("rad");
	max_kick_prop.set_format("%5.4f");
	max_kick_prop.set_description("Maximum magnet kick in radians.");
	max_kick->set_default_properties(max_kick_prop);
	att_list.push_back(max_kick);

	//	End of Automatic code generation
	//-------------------------------------------------------------
}

//+----------------------------------------------------------------------------
//
// method : 		ABendingClass::get_class_property()
// 
// description : 	Read the class properties from database.
//
//-----------------------------------------------------------------------------
void ABendingClass::get_class_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read class properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	cl_prop.push_back(Tango::DbDatum("DbName"));
	cl_prop.push_back(Tango::DbDatum("DbUser"));
	cl_prop.push_back(Tango::DbDatum("DbPassword"));
	cl_prop.push_back(Tango::DbDatum("DbHost"));
	cl_prop.push_back(Tango::DbDatum("ReadInterval"));
	cl_prop.push_back(Tango::DbDatum("MadSignConvention"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_class()->get_property(cl_prop);
	Tango::DbDatum	def_prop;
	int	i = -1;

	//	Try to extract DbUser value
	if (cl_prop[++i].is_empty()==false)	cl_prop[i]  >>  dbUser;
	else
	{
		//	Check default value for DbUser
		def_prop = get_default_class_property(cl_prop[i].name);
		if (def_prop.is_empty()==false)
		{
			def_prop  >>  dbName;
			cl_prop[i]  <<  dbName;
		}
		//	Check default value for DbUser
		def_prop = get_default_class_property(cl_prop[i].name);
		if (def_prop.is_empty()==false)
		{
			def_prop  >>  dbUser;
			cl_prop[i]  <<  dbUser;
		}
	}

	//	Try to extract DbPassword value
	if (cl_prop[++i].is_empty()==false)	cl_prop[i]  >>  dbPassword;
	else
	{
		//	Check default value for DbPassword
		def_prop = get_default_class_property(cl_prop[i].name);
		if (def_prop.is_empty()==false)
		{
			def_prop  >>  dbPassword;
			cl_prop[i]  <<  dbPassword;
		}
	}

	//	Try to extract DbHost value
	if (cl_prop[++i].is_empty()==false)	cl_prop[i]  >>  dbHost;
	else
	{
		//	Check default value for DbHost
		def_prop = get_default_class_property(cl_prop[i].name);
		if (def_prop.is_empty()==false)
		{
			def_prop  >>  dbHost;
			cl_prop[i]  <<  dbHost;
		}
	}

	//	Try to extract ReadInterval value
	if (cl_prop[++i].is_empty()==false)	cl_prop[i]  >>  readInterval;
	else
	{
		//	Check default value for ReadInterval
		def_prop = get_default_class_property(cl_prop[i].name);
		if (def_prop.is_empty()==false)
		{
			def_prop  >>  readInterval;
			cl_prop[i]  <<  readInterval;
		}
	}

	//	Try to extract MadSignConvention value
	if (cl_prop[++i].is_empty()==false)	cl_prop[i]  >>  madSignConvention;
	else
	{
		//	Check default value for MadSignConvention
		def_prop = get_default_class_property(cl_prop[i].name);
		if (def_prop.is_empty()==false)
		{
			def_prop  >>  madSignConvention;
			cl_prop[i]  <<  madSignConvention;
		}
	}

	//	End of Automatic code generation
	//------------------------------------------------------------------

}

//+----------------------------------------------------------------------------
//
// method : 	ABendingClass::set_default_property
// 
// description: Set default property (class and device) for wizard.
//              For each property, add to wizard property name and description
//              If default value has been set, add it to wizard property and
//              store it in a DbDatum.
//
//-----------------------------------------------------------------------------
void ABendingClass::set_default_property()
{
	string	prop_name;
	string	prop_desc;
	string	prop_def;

	vector<string>	vect_data;
	//	Set Default Class Properties

	prop_name = "DbName";
	prop_desc = "The MySQL magnet database name";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		cl_def_prop.push_back(data);
		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_class_prop(prop_name, prop_desc);

	prop_name = "DbUser";
	prop_desc = "The MySQL database user name";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		cl_def_prop.push_back(data);
		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_class_prop(prop_name, prop_desc);

	prop_name = "DbPassword";
	prop_desc = "The MySQL password";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		cl_def_prop.push_back(data);
		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_class_prop(prop_name, prop_desc);

	prop_name = "DbHost";
	prop_desc = "The Database host name";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		cl_def_prop.push_back(data);
		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_class_prop(prop_name, prop_desc);

	prop_name = "ReadInterval";
	prop_desc = "minimum time interval between two reading of the excitation. In milliseconds.\nThe purpose is to limit the number of calls to the real hardware";
	prop_def  = "500";
	vect_data.clear();
	vect_data.push_back("500");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		cl_def_prop.push_back(data);
		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_class_prop(prop_name, prop_desc);

	prop_name = "MadSignConvention";
	prop_desc = "Specify which sign convention to use for horizaontally focussing elements\nif True : focussing -> K>0 ( MAD, elegant,... )\nif False : focussing -> K<0 ( vicky )";
	prop_def  = "true";
	vect_data.clear();
	vect_data.push_back("true");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		cl_def_prop.push_back(data);
		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_class_prop(prop_name, prop_desc);

	//	Set Default Device Properties
	prop_name = "DbHost";
	prop_desc = "The Database host name";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);

	prop_name = "DbPassword";
	prop_desc = "The MySQL password";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);

	prop_name = "DbUser";
	prop_desc = "The MySQL database user name";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);

	prop_name = "MagnetName";
	prop_desc = "The name of the magnet, for instance `QD_B'\nThe name is used for database loopkup for getting calibration tables.";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);

	prop_name = "MagnetType";
	prop_desc = "The type of the magnet, must be one of the following:\nDIPOLE\nSECTORDIPOLE";
	prop_def  = "DIPOLE";
	vect_data.clear();
	vect_data.push_back("DIPOLE");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);

	prop_name = "PowerSupply";
	prop_desc = "The name of the power supply associated to the magnet.\nFor instance `b/power_supply/psqd_b'";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);

	prop_name = "PSExcitationAttributeName";
	prop_desc = "This is the name of the attribute that returns the excitation of the power supply.\nFor instance `Current' or `Voltage'";
	prop_def  = "Current";
	vect_data.clear();
	vect_data.push_back("Current");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);

	prop_name = "ReadInterval";
	prop_desc = "minimum time interval between two reading of the excitation. In milliseconds.\nThe purpose is to limit the number of calls to the real hardware";
	prop_def  = "500";
	vect_data.clear();
	vect_data.push_back("500");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);

	prop_name = "MadSignConvention";
	prop_desc = "Specify which sign convention to use for horizaontally focussing elements\nif True : focussing -> K>0 ( MAD, elegant,... )\nif False : focussing -> K<0 ( vicky )";
	prop_def  = "true";
	vect_data.clear();
	vect_data.push_back("true");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);

}
//+----------------------------------------------------------------------------
//
// method : 		ABendingClass::write_class_property
// 
// description : 	Set class description as property in database
//
//-----------------------------------------------------------------------------
void ABendingClass::write_class_property()
{
	//	First time, check if database used
	//--------------------------------------------
	if (Tango::Util::_UseDb == false)
		return;

	Tango::DbData	data;
	string	classname = get_name();
	string	header;
	string::size_type	start, end;

	//	Put title
	Tango::DbDatum	title("ProjectTitle");
	string	str_title("Class representing particel accelerator bending");
	title << str_title;
	data.push_back(title);

	//	Put Description
	Tango::DbDatum	description("Description");
	vector<string>	str_desc;
	str_desc.push_back("Tango device for manipulating accelerator magnets in terms of machine physics quantities.");
	str_desc.push_back("The SETTING of the excitation drives all the conversions and readings.");
	str_desc.push_back("That is: noise and fluctutations of the read-out are ignored.");
	str_desc.push_back("The magnet model is taken from Elettra's Hlf class library.");
	description << str_desc;
	data.push_back(description);
		
	//	put cvs or svn location
	string	filename(classname);
	filename += "Class.cpp";
	
	// Create a string with the class ID to
	// get the string into the binary
	string	class_id(ClassId);
	
	// check for cvs information
	string	src_path(CvsPath);
	start = src_path.find("/");
	if (start!=string::npos)
	{
		end   = src_path.find(filename);
		if (end>start)
		{
			string	strloc = src_path.substr(start, end-start);
			//	Check if specific repository
			start = strloc.find("/cvsroot/");
			if (start!=string::npos && start>0)
			{
				string	repository = strloc.substr(0, start);
				if (repository.find("/segfs/")!=string::npos)
					strloc = "ESRF:" + strloc.substr(start, strloc.length()-start);
			}
			Tango::DbDatum	cvs_loc("cvs_location");
			cvs_loc << strloc;
			data.push_back(cvs_loc);
		}
	}
	// check for svn information
	else
	{
		string	src_path(SvnPath);
		start = src_path.find("://");
		if (start!=string::npos)
		{
			end = src_path.find(filename);
			if (end>start)
			{
				header = "$HeadURL: ";
				start = header.length();
				string	strloc = src_path.substr(start, (end-start));
				
				Tango::DbDatum	svn_loc("svn_location");
				svn_loc << strloc;
				data.push_back(svn_loc);
			}
		}
	}

	//	Get CVS or SVN revision tag
	
	// CVS tag
	string	tagname(TagName);
	header = "$Name: ";
	start = header.length();
	string	endstr(" $");
	
	end   = tagname.find(endstr);
	if (end!=string::npos && end>start)
	{
		string	strtag = tagname.substr(start, end-start);
		Tango::DbDatum	cvs_tag("cvs_tag");
		cvs_tag << strtag;
		data.push_back(cvs_tag);
	}
	
	// SVN tag
	string	svnpath(SvnPath);
	header = "$HeadURL: ";
	start = header.length();
	
	end   = svnpath.find(endstr);
	if (end!=string::npos && end>start)
	{
		string	strloc = svnpath.substr(start, end-start);
		
		string tagstr ("/tags/");
		start = strloc.find(tagstr);
		if ( start!=string::npos )
		{
			start = start + tagstr.length();
			end   = strloc.find(filename);
			string	strtag = strloc.substr(start, end-start-1);
			
			Tango::DbDatum	svn_tag("svn_tag");
			svn_tag << strtag;
			data.push_back(svn_tag);
		}
	}

	//	Get URL location
	string	httpServ(HttpServer);
	if (httpServ.length()>0)
	{
		Tango::DbDatum	db_doc_url("doc_url");
		db_doc_url << httpServ;
		data.push_back(db_doc_url);
	}

	//  Put inheritance
	Tango::DbDatum	inher_datum("InheritedFrom");
	vector<string> inheritance;
	inheritance.push_back("TANGO_BASE_CLASS");
	inher_datum << inheritance;
	data.push_back(inher_datum);

	//	Call database and and values
	//--------------------------------------------
	get_db_class()->put_property(data);
}

}	// namespace
